/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* cube-server
 *
 * Copyright (C) Jake R. Capangpangan 2015 <camicrisystems@gmail.com>
 *
cube-server is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cube-server is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using Gee;

public class ServerCommandConfiguration : GLib.Object {

    public static string cmd_get_configuration_entries ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;
		string config_entry = """ "%s" : "%s" """;

		int index = 0;
		foreach ( string key in css.cube.base_manager.main_configuration_file.configuration_map.keys )
		{
			string val = css.cube.base_manager.main_configuration_file.configuration_map[key];
			
			json += (index>0?",":"") + config_entry.printf(key.replace("-","_"),val);

			index++;
		}

		return " { " + json + " } ";
	}

	public static string cmd_save_configuration_entries ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s", "message" : "%s" } """;

		foreach ( string key in get_data_map.keys )
		{
			string config_key = key.replace("_","-");

			if ( css.cube.base_manager.main_configuration_file.configuration_map.has_key(config_key) )
			{				
				string val = get_data_map[key];				
				if ( !css.cube.base_manager.main_configuration_file.set_value(config_key,val) )
					return json.printf("false","Failed to set configuration entry for '"+config_key+"'");
			}
		}

		bool result = css.cube.base_manager.main_configuration_file.save_configuration();

		if ( result )
		{
			if ( !css.cube.base_manager.main_configuration_file.open_configuration() )
				json = json.printf("false","Failed to re-open configuration file");
			else
				json = json.printf("true","Configuration Saved");
		}
		else
			json = json.printf("false","Failed to save configuraiton changes");

		return json;
	}

	public static string cmd_open_sources_list_file ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s", "message" : "%s" } """ ;

		int res = -1;
		
		if ( SystemInformation.get_operating_system_type() == OperatingSystemType.LINUX )
			res = ProcessManager.run_get_status ({"gedit",css.cube.project.sources_list_file});
		else if ( SystemInformation.get_operating_system_type() == OperatingSystemType.WINDOWS )
			res = ProcessManager.run_get_status ({"explorer",css.cube.project.sources_list_file});

		if ( res == 0 )
			json = json.printf("true","Successfully opened sources list file");
		else
			json = json.printf("false","Failed to open sources list file");
		
		return json;
	}
}
