/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* cube-server
 *
 * Copyright (C) Jake R. Capangpangan 2015 <camicrisystems@gmail.com>
 *
cube-server is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cube-server is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using Gee;

public class ServerCommandDownload : GLib.Object {

	public static string cmd_download_package ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s" , "message" : "%s" } """;

		if ( css.flag_status_map["package_downloading"] )
			return json.printf("false","Ongoing Download");

		string[] packages = null;		

		if ( get_data_map.has_key("package") )
		{		
			if ( get_data_map["package"].contains(";") )
				packages = get_data_map["package"].split(";",-1);
			else
				packages = new string[]{get_data_map["package"]};
		}
		else
			return json.printf("false","");
		

		ArrayList<Package> pkgs = new ArrayList<Package>();
		foreach ( string name in packages )
		{
			if ( css.cube.repository_manager.available_packages.has_key(name) )
			{
				Package p = css.cube.repository_manager.available_packages[name];			
				if ( !pkgs.contains(p) )
					pkgs.add(p);
			}
		}

		ArrayList<Package> unsatisfied_dependencies = new ArrayList<Package>();
		ArrayList<Package> satisfied_dependencies = new ArrayList<Package>();
		ArrayList<Package> dependency_list = new ArrayList<Package>();
		
		foreach ( Package p in pkgs )
		{
			ArrayList<Package> curr_unsatisfied_dep = new ArrayList<Package>();			
			
			css.cube.get_package_dependencies(p,curr_unsatisfied_dep,satisfied_dependencies,dependency_list);
			
			foreach ( Package p_curr in curr_unsatisfied_dep )
			{
				if ( !unsatisfied_dependencies.contains(p_curr) )
					unsatisfied_dependencies.add(p_curr);
			}			
		}

		bool res = css.thread_download_packages(unsatisfied_dependencies);
		
		return json.printf(res.to_string(),"Download Started");
	}

	public static string cmd_download_repository ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s" , "message" : "%s" } """;		

		string? ppa = null;

		if ( get_data_map.has_key("ppa") )
			ppa = get_data_map["ppa"];
		
		bool res = css.thread_download_repositories(ppa);
		
		return json.printf(res.to_string(),"Download Started");
	}

	public static string cmd_download_stop ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s" , "message" : "%s" } """;		

		bool res = css.download_stop();
		
		return json.printf(res.to_string(),"Download Stopped");
	}

	public static string cmd_get_download_status ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "current_index" : "%d", "current_item" : "%s" , "current_size" : "%s" ,"current_progress" : "%d", "current_transfer_rate" : "%s" , "overall_progress_success" : "%d" , "overall_progress_failed" : "%d" , "max_count" : "%d" } """;

		if ( css.download_status == null )
			return "";

		if ( css.download_status.index == -1 )
			return "";
		
		if ( css.download_status.download_type == DownloadType.PACKAGE )
		{			
			json = json.printf(
			            css.download_status.index + 1 ,
			            css.download_status.packages[css.download_status.index].name,
			            SizeConverter.convert(css.download_status.packages[css.download_status.index].size),
				        css.download_status.progress[css.download_status.index],
			            css.download_status.transfer_rate[css.download_status.index],
				        css.download_status.overall_progress_success,
			            css.download_status.overall_progress_failed,
			            css.download_status.packages.length
				        );
		}
		else
		{			
			
			string repo_name = css.download_status.sources[css.download_status.index].ppa + " "  
				+ css.download_status.sources[css.download_status.index].release + " "
				+ css.download_status.sources[css.download_status.index].component;

			json = json.printf(
			            css.download_status.index + 1,
			                   repo_name,			            
			            SizeConverter.convert(css.download_status.size[css.download_status.index]),
				        css.download_status.progress[css.download_status.index],
			            css.download_status.transfer_rate[css.download_status.index],
				        css.download_status.overall_progress_success,
			            css.download_status.overall_progress_failed,
			            css.download_status.sources.length
			);
		}

		return json;
	}

	public static string cmd_get_download_package_result ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : [%s] , "failed" : [%s] } """;
		string success = "";
		string failed = "";
		string download_item = """ { "name" : "%s" , "size" : "%s" , "error_message" : "%s" } """;		

		if ( css.download_status == null )
			return json.printf("","");
		if ( css.download_status.packages == null )
			return json.printf("","");

		for ( int i = 0; i < css.download_status.packages.length; i++ )
		{
			string name = css.download_status.packages[i].name;
			string size = css.download_status.size[i];
			string error_message = css.download_status.error_message[i];
			
			if ( css.download_status.status_type[i] == DownloadStatusType.FINISHED )
				success += (success!=""?",":"") + download_item.printf(name,size,error_message);
			else
				failed += (failed!=""?",":"") + download_item.printf(name,size,error_message);
		}

		return json.printf(success,failed);
	}

	public static string cmd_get_download_repository_result ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : [%s] , "failed" : [%s] } """;
		string success = "";
		string failed = "";
		string download_item = """ { "name" : "%s" , "size" : "%s" , "error_message" : "%s" } """;		

		if ( css.download_status == null )
			return json.printf("","");
		if ( css.download_status.sources == null )
			return json.printf("","");
		
		for ( int i = 0; i < css.download_status.sources.length; i++ )
		{
			string name = css.download_status.sources[i].ppa + " "  
				+ css.download_status.sources[i].release + " "
				+ css.download_status.sources[i].component;
						
			string size = css.download_status.size[i];
			string error_message = css.download_status.error_message[i];
			
			if ( css.download_status.status_type[i] == DownloadStatusType.FINISHED )
				success += (success!=""?",":"") + download_item.printf(name,size,error_message);
			else
				failed += (failed!=""?",":"") + download_item.printf(name,size,error_message);
		}

		return json.printf(success,failed);
	}
}
