/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* cube-server
 *
 * Copyright (C) Jake R. Capangpangan 2015 <camicrisystems@gmail.com>
 *
cube-server is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cube-server is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using Gee;

public class ServerCommandInstall : GLib.Object {
	
	public static string cmd_install_package ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s" , "message" : "%s" } """;
		string[] packages;

		if ( css.flag_status_map["package_installing"] )
			return json.printf("false","Ongoing Installation");
		
		if ( get_data_map["package"].contains(";") )
			packages = get_data_map["package"].split(";",-1);
		else
			packages = new string[]{get_data_map["package"]};

		ArrayList<Package> to_install = new ArrayList<Package>();
		ArrayList<Package> to_download = new ArrayList<Package>();
		ArrayList<Package> dependency_list = new ArrayList<Package>();

		foreach ( string name in packages )
		{
			if ( !css.cube.repository_manager.available_packages.has_key(name) )
				continue;
			
			Package p_curr = css.cube.repository_manager.available_packages[name];
			ArrayList<Package> satisfied = new ArrayList<Package>();
			ArrayList<Package> unsatisfied = new ArrayList<Package>();

			css.cube.get_package_dependencies(p_curr,unsatisfied,satisfied,dependency_list);
			foreach ( Package p in unsatisfied )
			{
				if ( !to_download.contains(p) )
				{
					to_download.add(p);
					stdout.printf("To Download: %s\n",p.name);
				}
			}
			foreach ( Package p in satisfied )
			{
				if ( !to_install.contains(p) )
				{
					to_install.add(p);
					stdout.printf("To Install: %s\n",p.name);
				}
			}
		}		

		if ( to_install.size > 0 && to_download.size == 0 )
		{			
			css.thread_install_packages(to_install);
			return json.printf("true","Installation Started");
		}
		else
			return json.printf("false","Installation Aborted. There are still packages to download");		
	}

	public static string cmd_get_install_package_result ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s" , "message" : "%s" } """;
		string success = css.flag_result_map["install_success"];
		string message = css.flag_result_map["install_message"];		

		return json.printf(success,message);
	}
}
