/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* cube-server
 *
 * Copyright (C) Jake R. Capangpangan 2015 <camicrisystems@gmail.com>
 *
cube-server is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cube-server is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using Gee;

public class ServerCommandProject : GLib.Object {

	public static string cmd_get_projects ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = "";

		string project_item = """{ "name" : "%s", "version" : "%s", "computer_name" : "%s" , "operating_system" : "%s", "distribution" : "%s", "release" : "%s", "codename" : "%s", "upstream_distribution" : "%s", "upstream_release" : "%s", "upstream_codename" : "%s", "architecture" : "%s", "date_created" : "%s" }""";

		int index = 0;
		css.cube.project_manager.get_projects();
		
		foreach ( Project p in css.cube.project_manager.all_projects )
		{
			json += ((index>0)?",":"") + project_item.printf(p.project_name, p.project_version, p.computer_name, p.operating_system, p.distribution, p.release, p.codename, p.upstream_distribution, p.upstream_release, p.upstream_codename, p.architecture, p.date_created );
			index++;
		}

		return "[" + json + "]";
	}

	public static string cmd_get_current_project ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """{ "name" : "%s", "version" : "%s", "computer_name" : "%s" , "operating_system" : "%s", "distribution" : "%s", "release" : "%s", "codename" : "%s", "upstream_distribution" : "%s", "upstream_release" : "%s", "upstream_codename" : "%s", "architecture" : "%s", "date_created" : "%s" }""";

		if ( css.cube.project == null )
			return "";

		return json.printf(css.cube.project.project_name, css.cube.project.project_version, css.cube.project.computer_name, css.cube.project.operating_system, css.cube.project.distribution, css.cube.project.release, css.cube.project.codename, css.cube.project.upstream_distribution, css.cube.project.upstream_release, css.cube.project.upstream_codename, css.cube.project.architecture, css.cube.project.date_created );
	}
	
    public static string cmd_create_project ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s" , "message" : "%s" } """;
		
		if ( get_data_map.has_key ("project") )
		{
			if ( get_data_map["project"].strip() != "" && !get_data_map["project"].strip().contains(" ") )
			{
				if ( css.create_project( get_data_map["project"] ) )
					json = json.printf("true", "Project %s created".printf(get_data_map["project"]) );
				else
					json = json.printf("false", css.error_message );
			}
			else
				json = json.printf("false", "Invalid project name");
		}
		else
			json = json.printf("false","No project name received");

		return json;
	}

	public static string cmd_open_project ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s" , "project" : "%s", "message" : "%s" } """;

		if ( !get_data_map.has_key ("project") )
			return json.printf("false","null","No project name provided");

		if ( !(get_data_map["project"].strip() != "" && !get_data_map["project"].strip().contains(" ")) )
			return json.printf("false","null","Invalid project name");

		if ( !css.open_project (get_data_map["project"]) )
			return json.printf("false",get_data_map["project"],css.error_message);

		return json.printf("true",get_data_map["project"],"Project Opened");
	}

	public static string cmd_open_projects_directory ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s" , "message" : "%s" } """;
		
		bool result = ProcessManager.run_default_application ( css.cube.base_manager.projects_directory );

		return json.printf(result.to_string(),"");
	}
	
}
