/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* cube-server
 *
 * Copyright (C) Jake R. Capangpangan 2015 <camicrisystems@gmail.com>
 *
cube-server is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cube-server is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using Gee;

public class ServerCommandRepository : GLib.Object {

	public static string cmd_scan_repositories ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{		
		string json = """ { "success" : "%s" , "message" : "%s" } """;
		bool status_file_only = false;

		if ( css.flag_status_map["repository_scanning"] )
			return json.printf("false","Ongoing Repository Scan");

		if ( get_data_map.has_key("mode") )
		{
			stdout.printf("Status file scan only\n");
			if ( get_data_map["mode"] == "status-file-only" )
				status_file_only = true;
		}
		
		css.thread_scan_repositories(status_file_only);
		return json.printf("true","Scan repository thread started");
	}	
	
	public static string cmd_get_main_lists ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;

		string main_list_item = """{ "name" : "%s" , "description" : "%s" , "count" : "%d" }""";

		json += main_list_item.printf("available","Available Packages",css.cube.repository_manager.available_packages_count) + ",";
		json += main_list_item.printf("installed","Installed Packages",css.cube.repository_manager.installed_packages_count) + ",";
		json += main_list_item.printf("upgradable","Upgradable Packages",css.cube.repository_manager.upgradable_packages_count) + ",";
		json += main_list_item.printf("downloaded","Downloaded Packages",css.cube.repository_manager.downloaded_packages_count) + ",";		
		json += main_list_item.printf("marked-to-download","Marked to Download Packages",css.marked_to_download_packages.size) + ",";
		json += main_list_item.printf("marked-to-install","Marked to Installed Packages",css.marked_to_install_packages.size);
		
		return "[" + json + "]";
	}

	

	public static string cmd_get_sections ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;

		string section_item = """{ "name" : "%s" , "description" : "%s" , "count" : "%d" }""";

		int index = 0;
		foreach ( string key in css.cube.repository_manager.section_map.keys )
		{
			int count = css.cube.repository_manager.section_map[key].size;
			string description = css.cube.base_manager.section_configuration_file.get_value(key);
			json += ((index>0)?",":"") + section_item.printf(key,description,count);
			index++;
		}

		return "[" + json + "]";
	}



	public static string cmd_get_next ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;
		string package_item = """ { "name" : "%s", "version" : "%s", "description" : "%s", "status" : "%s", "section" : "%s", "size" : "%s", "marked" : "%s" } """;

		ArrayList<Package> packages = new ArrayList<Package>();
		int count = 10;

		if ( !get_data_map.has_key("type") )
			return json;

		if ( get_data_map.has_key("count") )
			count = int.parse(get_data_map["count"]);
		else
			count = 10;
		
		switch ( get_data_map["type"] )
		{
			case "main" :
				packages = css.package_query.next(count);
				break;
			case "current" :				
				packages = css.current_query.next(count);
				break;
		}

		int index = 0;
		bool marked = false;
		foreach ( Package p in packages )
		{
			if ( p.status == PackageStatusType.DOWNLOADED )
				css.marked_to_install_packages.contains(p.name)?marked=true:marked=false;
			else if ( p.status != PackageStatusType.INSTALLED )
				css.marked_to_download_packages.contains(p.name)?marked=true:marked=false;
			
			json += ((index>0)?",":"") + package_item.printf(p.name,p.version,p.description.replace("\\","\\\\").replace("\"","\\\""),get_package_status_string(p),p.section,SizeConverter.convert(p.size),marked.to_string());
			index++;
		}

		return "[" + json + "]";
	}


	
	public static string cmd_get_previous ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;
		string package_item = """ { "name" : "%s", "version" : "%s", "description" : "%s", "status" : "%s", "section" : "%s", "size" : "%s", "marked" : "%s" } """;

		ArrayList<Package> packages = new ArrayList<Package>();
		int count = 10;

		if ( !get_data_map.has_key("type") )
			return json;

		if ( get_data_map.has_key("count") )
			count = int.parse(get_data_map["count"]);
		else
			count = 10;
		
		switch ( get_data_map["type"] )
		{
			case "main" :
				packages = css.package_query.previous(count);
				break;
			case "current" :
				packages = css.current_query.previous(count);
				break;
		}

		int index = 0;
		bool marked = false;
		foreach ( Package p in packages )
		{
			if ( p.status == PackageStatusType.DOWNLOADED )
				css.marked_to_install_packages.contains(p.name)?marked=true:marked=false;
			else if ( p.status != PackageStatusType.INSTALLED )
				css.marked_to_download_packages.contains(p.name)?marked=true:marked=false;
			
			json += ((index>0)?",":"") + package_item.printf(p.name,p.version,p.description.replace("\\","\\\\").replace("\"","\\\""),get_package_status_string(p),p.section,SizeConverter.convert(p.size),marked.to_string());
			index++;
		}

		return "[" + json + "]";
	}


	
	public static string cmd_get_section_packages ( CubeServerSystem css , TreeMap<string,string> get_data_map )
	{
		string json = """ """;

		if ( ! css.cube.repository_manager.section_map.has_key( get_data_map["section"] ) )
		    return json;

		css.current_query = new PackageQuery.package_list (css.cube.repository_manager.section_map[get_data_map["section"]],PackageQueryType.SOME);
		css.current_query.index = 0;

		get_data_map["type"] = "current";
		get_data_map["count"] = "20";
		
		return cmd_get_next (css,get_data_map); 		
	}


	
	public static string cmd_get_available_packages ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;		

		css.current_query = css.package_query;
		css.current_query.index = 0;

		get_data_map["type"] = "current";
		get_data_map["count"] = "20";
		
		return cmd_get_next (css,get_data_map); 
	}


	
	public static string cmd_get_installed_packages ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;		

		css.current_query = new PackageQuery.package_map (css.cube.repository_manager.installed_packages,PackageQueryType.SOME);
		css.current_query.index = 0;

		get_data_map["type"] = "current";
		get_data_map["count"] = "20";
		
		return cmd_get_next (css,get_data_map); 
	}


	
	public static string cmd_get_upgradable_packages ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;

		ArrayList<Package> upgradable_packages = new ArrayList<Package>();
		foreach ( string name in css.cube.repository_manager.upgradable_package_names )
			upgradable_packages.add(css.cube.repository_manager.available_packages[name]);

		css.current_query = new PackageQuery.package_list ( upgradable_packages, PackageQueryType.SOME );
		css.current_query.index = 0;

		get_data_map["type"] = "current";
		get_data_map["count"] = "20";
		
		return cmd_get_next (css,get_data_map); 
	}


	
	public static string cmd_get_downloaded_packages ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;

		ArrayList<Package> downloaded_packages = new ArrayList<Package>();
		foreach ( string name in css.cube.repository_manager.downloaded_package_names )
			downloaded_packages.add(css.cube.repository_manager.available_packages[name]);

		css.current_query = new PackageQuery.package_list ( downloaded_packages, PackageQueryType.SOME );
		css.current_query.index = 0;

		get_data_map["type"] = "current";
		get_data_map["count"] = "20";
		
		return cmd_get_next (css,get_data_map); 
	}



	public static string cmd_get_package_dependencies ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "satisfied" : [ %s ] , "unsatisfied" : [ %s ] , "satisfied_size" : "%s", "unsatisfied_size" : "%s", "overall_size" : "%s" }""";
		string package_item = """ { "name" : "%s", "version" : "%s", "description" : "%s", "status" : "%s", "section" : "%s", "size" : "%s" } """;

		string satisfied_package_items = "";
		string unsatisfied_package_items = "";
		double satisfied_package_items_size = 0;
		double unsatisfied_package_items_size = 0;

		string[] packages = null;		

		if ( get_data_map.has_key("package") )
		{		
			if ( get_data_map["package"].contains(";") )
				packages = get_data_map["package"].split(";",-1);
			else
				packages = new string[]{get_data_map["package"]};
		}

		if ( get_data_map.has_key("type") )
		{
			if ( get_data_map["type"] == "all-marked-to-download" )
				packages = css.marked_to_download_packages.to_array();
			else if ( get_data_map["type"] == "all-marked-to-install" )
				packages = css.marked_to_install_packages.to_array();
		}		

		ArrayList<Package> pkgs = new ArrayList<Package>();
		foreach ( string name in packages )
		{
			if ( css.cube.repository_manager.available_packages.has_key(name) )
			{
				Package p = css.cube.repository_manager.available_packages[name];
				if ( !pkgs.contains(p) )
					pkgs.add(p);
			}
		}

		ArrayList<Package> unsatisfied_dependencies = new ArrayList<Package>();
		ArrayList<Package> satisfied_dependencies = new ArrayList<Package>();
		ArrayList<Package> dependency_list = new ArrayList<Package>();
		
		foreach ( Package p in pkgs )
		{
			stdout.printf("Package : %s\n",p.name);
			ArrayList<Package> curr_unsatisfied_dep = new ArrayList<Package>();
			ArrayList<Package> curr_satisfied_dep = new ArrayList<Package>();
			
			css.cube.get_package_dependencies(p,curr_unsatisfied_dep,curr_satisfied_dep,dependency_list);
			
			foreach ( Package p_curr in curr_unsatisfied_dep )
			{
				if ( !unsatisfied_dependencies.contains(p_curr) )
					unsatisfied_dependencies.add(p_curr);
			}

			foreach ( Package p_curr in curr_satisfied_dep )
			{
				if ( !satisfied_dependencies.contains(p_curr) )
					satisfied_dependencies.add(p_curr);
			}
		}

		satisfied_dependencies.sort(Package.compare_function);
		unsatisfied_dependencies.sort(Package.compare_function);		

		int index = 0;
		foreach ( Package p in unsatisfied_dependencies )
		{
			unsatisfied_package_items += ((index>0)?",":"") + package_item.printf(p.name,p.version,p.description.replace("\\","\\\\").replace("\"","\\\""),get_package_status_string(p),p.section,SizeConverter.convert(p.size));
			unsatisfied_package_items_size += double.parse(p.size);
			index++;
		}

		index = 0;
		foreach ( Package p in satisfied_dependencies )
		{
			satisfied_package_items += ((index>0)?",":"") + package_item.printf(p.name,p.version,p.description.replace("\\","\\\\").replace("\"","\\\""),get_package_status_string(p),p.section,SizeConverter.convert(p.size));
			satisfied_package_items_size += double.parse(p.size);
			index++;
		}
		
		return json.printf(satisfied_package_items,
		                   unsatisfied_package_items,
		                   SizeConverter.convert(satisfied_package_items_size.to_string()),
		                   SizeConverter.convert(unsatisfied_package_items_size.to_string()),
		                   SizeConverter.convert((satisfied_package_items_size+unsatisfied_package_items_size).to_string())
		                   );
	}

	public static string cmd_get_satisfied_downloaded_packages( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;
		string package_item = """ { "name" : "%s", "version" : "%s", "description" : "%s", "status" : "%s", "section" : "%s", "size" : "%s" } """;

		ArrayList<Package> satisfied_downloaded_packages = new ArrayList<Package>();
		foreach ( string name in css.cube.repository_manager.downloaded_package_names)
		{
			Package p = css.cube.repository_manager.available_packages[name];
			ArrayList<Package> unsatisfied_dependencies = new ArrayList<Package>();
			
			css.cube.get_package_dependencies(p,unsatisfied_dependencies);

			if ( unsatisfied_dependencies.size == 0 )
				satisfied_downloaded_packages.add(p);
		}

		satisfied_downloaded_packages.sort(Package.compare_function);

		int index = 0;
		foreach ( Package p in satisfied_downloaded_packages )
		{
			json += ((index>0)?",":"") + package_item.printf(p.name,p.version,p.description.replace("\\","\\\\").replace("\"","\\\""),get_package_status_string(p),p.section,SizeConverter.convert(p.size));
			index++;
		}

		return " [ " + json + " ] ";
	}

	
	public static string cmd_find_package ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;

		string package_item = """ { "name" : "%s", "version" : "%s", "description" : "%s", "status" : "%s", "section" : "%s", "size" : "%s", "marked" : "%s" } """;

		string[] packages;
		if ( get_data_map["package"].contains(";") )
			packages = get_data_map["package"].split(";",-1);
		else
			packages = new string[]{get_data_map["package"]};

		bool first = true;
		bool marked = false;
		foreach ( string name in packages )
		{
			Package p = css.package_query.find_package ( name );
			if ( p != null )
			{
				if ( p.status == PackageStatusType.DOWNLOADED )
					css.marked_to_install_packages.contains(p.name)?marked=true:marked=false;
				else if ( p.status != PackageStatusType.INSTALLED )
					css.marked_to_download_packages.contains(p.name)?marked=true:marked=false;
				
				json += (first?"":",") + package_item.printf(p.name,p.version,p.description.replace("\\","\\\\").replace("\"","\\\""),get_package_status_string(p),p.section,SizeConverter.convert(p.size),marked.to_string());
				first = false;
			}
		}		
		
		return "[" + json + "]";
	}


	
	public static string cmd_find_package_starts_with ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ """;

		string package_item = """ { "name" : "%s", "version" : "%s", "description" : "%s", "status" : "%s", "section" : "%s", "size" : "%s", "marked" : "%s" } """;

		int index = 0;
		css.current_query.index = 0;

		bool marked = false;
		foreach ( Package p in css.current_query.find_starts_with ( get_data_map["package"] , 20 ) )
		{
			if ( p.status == PackageStatusType.DOWNLOADED )
				css.marked_to_install_packages.contains(p.name)?marked=true:marked=false;
			else if ( p.status != PackageStatusType.INSTALLED )
				css.marked_to_download_packages.contains(p.name)?marked=true:marked=false;
			
			json += ((index>0)?",":"") + package_item.printf(p.name,p.version,p.description.replace("\\","\\\\").replace("\"","\\\""),get_package_status_string(p),p.section,SizeConverter.convert(p.size),marked.to_string());
			index++;
		}

		return "[" + json + "]";
	}

	public static string cmd_get_marked_packages ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{	
		if ( !get_data_map.has_key("type") )
			return "";

		ArrayList<string> package_name_list = new ArrayList<string>();
		ArrayList<Package> marked_packages = new ArrayList<Package>();
		
		if ( get_data_map["type"] == "to-install" )
			package_name_list = css.marked_to_install_packages;
		else if ( get_data_map["type"] == "to-download" )
			package_name_list = css.marked_to_download_packages;

		foreach ( string name in package_name_list )
			marked_packages.add(css.cube.repository_manager.available_packages[name]);

		css.current_query = new PackageQuery.package_list ( marked_packages, PackageQueryType.SOME );
		css.current_query.index = 0;

		//Override for get next
		get_data_map["type"] = "current";
		if ( !get_data_map.has_key("count") )
			get_data_map["count"] = "20";
		
		stdout.printf(get_data_map["count"]+"\n");
		
		return cmd_get_next (css,get_data_map); 
	}	

	public static string cmd_mark_package ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s", "message" : "%s" } """;

		if ( !get_data_map.has_key("type") )
			return json.printf("false","");

		string?[] packages = null;
		MarkPackageType? type = null;
		
		if ( get_data_map["type"] == "all-downloaded" )
		{
			type = MarkPackageType.TO_INSTALL;
			packages = css.cube.repository_manager.downloaded_package_names.to_array();
		}
		else if ( get_data_map["type"] == "all-upgradable" )
		{
			type = MarkPackageType.TO_DOWNLOAD;
			packages = css.cube.repository_manager.upgradable_package_names.to_array();
		}
		else if ( get_data_map["type"] == "to-install" )
		{
			type = MarkPackageType.TO_INSTALL;
			if ( get_data_map.has_key("package") )
			{
				if ( get_data_map["package"].contains(";") )
					packages = get_data_map["package"].split(";",-1);
				else
					packages = new string[]{ get_data_map["package"] };
			}
		}
		else if ( get_data_map["type"] == "to-download" )
		{
			type = MarkPackageType.TO_DOWNLOAD;
			if ( get_data_map.has_key("package") )
			{
				if ( get_data_map["package"].contains(";") )
					packages = get_data_map["package"].split(";",-1);
				else
					packages = new string[]{ get_data_map["package"] };
			}
		}
		else
			json = json.printf("false","");

		if ( packages != null )
		{
			foreach ( string name in packages )
				css.mark_package(type,name);
			json = json.printf("true","");
		}
		else
			json = json.printf("false","");
		
		return json;
	}

	public static string cmd_unmark_package ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "success" : "%s", "message" : "%s" } """;

		if ( !get_data_map.has_key("type") )
			return json.printf("false","");

		if ( get_data_map["type"] == "all-to-download" )
		{
			css.clear_marked_packages(MarkPackageType.TO_DOWNLOAD);
			json = json.printf("true","");
		}
		else if ( get_data_map["type"] == "all-to-install" )
		{
			css.clear_marked_packages(MarkPackageType.TO_INSTALL);
			json = json.printf("true","");
		}
		else if ( get_data_map["type"] == "to-install" )
		{
			if ( get_data_map.has_key("package") )
			{
				css.unmark_package(MarkPackageType.TO_INSTALL,get_data_map["package"]);
				json = json.printf("true","");
			}
			else
				json = json.printf("false","");
		}
		else if ( get_data_map["type"] == "to-download" )
		{
			if ( get_data_map.has_key("package") )
			{
				css.unmark_package(MarkPackageType.TO_DOWNLOAD,get_data_map["package"]);
				json = json.printf("true","");
			}
			else
				json = json.printf("false","");
		}
		else
			json = json.printf("false","");

		return json;
	}
	
	private static string get_package_status_string ( Package p )
	{
		switch ( p.status )
		{
			case PackageStatusType.AVAILABLE : return "Available";
			case PackageStatusType.INSTALLED : return "Installed";
			case PackageStatusType.DOWNLOADED : return "Downloaded";
			case PackageStatusType.UPGRADABLE : return "Upgradable";
		}

		return "";
	}
	
}