/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* cube-server
 *
 * Copyright (C) Jake R. Capangpangan 2015 <camicrisystem@gmail.com>
 *
cube-server is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cube-server is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using Gee;

public class ServerCommandSystem : GLib.Object {

	public static string cmd_get_environment_data ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = """ { "operating_system" : "%s", "original_computer" : "%s" }  """;
		string os = "UNKNOWN";
		string original = "false";
		
		if ( SystemInformation.get_operating_system_type () == OperatingSystemType.LINUX )
			os = "LINUX";
		else if ( SystemInformation.get_operating_system_type () == OperatingSystemType.WINDOWS )
			os = "WINDOWS";
		
		if ( css.cube.project != null )
		{
			if ( css.cube.project.is_project_original_computer() )
				original = "true";		
		}

		return json.printf(os,original);
	}	

	public static string cmd_get_cube_data ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{		
		string json = """ { "application_name" : "%s", "application_version" : "%s", "application_revision" : "%s", "description" : "%s", "short_description" : "%s", "authors" : [%s], "translators" : [%s], "launchpad_url" : "%s", "facebook_url" : "%s", "license" : "%s" } """;
		string authors = "";
		string translators = "";

		int index = 0;
		foreach ( string author in CubeInformation.authors )
		{
			authors += (index!=0?",":"") + """ { "name" : "%s" } """.printf(author);
			index ++;
		}

		index = 0;
		foreach ( string translator in CubeInformation.translators )
		{
			translators += (index!=0?",":"") + """ { "name" : "%s" } """.printf(translator);
			index++;
		}

		return json.printf(CubeInformation.application_name,
		                   CubeInformation.application_version,
		                   CubeInformation.application_revision,
		                   CubeInformation.description,
		                   CubeInformation.short_description,
		                   authors,
		                   translators,
		                   CubeInformation.launchpad_url,
		                   CubeInformation.facebook_url,
		                   CubeInformation.license);		
	}
	
	public static string cmd_get_status ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = "";

		string status_item = """"%s" : "%s" """; 

		int index = 0;
		foreach ( string key in css.flag_status_map.keys )
		{			
			json += ((index>0)?",":"") + status_item.printf(key,css.flag_status_map[key].to_string());
			index++;
		}

		return "{ " + json + " }";
	}

	public static string cmd_get_result ( CubeServerSystem css, TreeMap<string,string> get_data_map )
	{
		string json = "";

		string status_item = """"%s" : "%s" """; 

		int index = 0;
		foreach ( string key in css.flag_result_map.keys )
		{			
			json += ((index>0)?",":"") + status_item.printf(key,css.flag_result_map[key]);
			index++;
		}

		return "{ " + json + " }";
	}
}
