/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* cube-server
 *
 * Copyright (C) Jake R. Capangpangan 2015 <camicrisystems@gmail.com>
 *
cube-server is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cube-server is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using Gee;

public class CubeServerRouter : GLib.Object {

	CubeServerSystem _css = new CubeServerSystem();	

	public CubeServerSystem server_system { get { return _css; } set { _css = value; } }

	public signal void response_occurred ( uint8[] content_data, int content_length, string type );
	public signal void stop_server_requested ();	

	public CubeServerRouter ( CubeServerSystem css )
	{
		_css = css;
	}
	
	public void route ( string location, TreeMap<string,string> get_data_map )
	{	
		if ( location.has_prefix("/cmd") )
			route_cmd ( location, get_data_map );
		else if ( location.has_prefix("/assets") )
			route_assets ( location, get_data_map );
		else
			route_page ( location, get_data_map );
	}	

	public void route_page ( string location, TreeMap<string,string> get_data_map )
	{
		string content = "";
		
		switch ( location )
		{	
			//ServerPageStartup
			case "/startup" :
				content = _css.view_headers("Startup");
				content += ServerPageStartup.view_startup ( _css, get_data_map );
				content += _css.view_footers();
				break;

			case "/startup/create-project" :
				content = _css.view_headers("Create Project");
				content += ServerPageStartup.view_create_project ( _css, get_data_map );
				content += _css.view_footers();
				break;

			case "/startup/delete-project" :
				content = _css.view_headers("Delete Project");
				content += ServerPageStartup.view_delete_project ( _css, get_data_map );
				content += _css.view_footers();
				break;	

			//ServerPageMain
			case "/" :
				/*
				content = _css.view_headers("Main");
				content += ServerPageMain.view_main ( _css, get_data_map );
				content += _css.view_footers();
				*/
				get_data_map["location"] = "localhost/main";
				content += ServerPages.page_any( _css,get_data_map );
				break;

			case "/list-package" :
				content = _css.view_headers("List Packages");
				content += ServerPageMain.view_list_package ( _css, get_data_map );
				content += _css.view_footers();
				break;

			default :
				content = _css.view_headers("Not Found");
				content += "<h1>404 Not found</h1>";
				content += _css.view_footers();
				break;
		}

		response_occurred ( content.data , content.data.length, "text/html" );
	}

	public void route_cmd ( string location, TreeMap<string,string> get_data_map )
	{
		string content = "";
		
		switch ( location )
		{
			//System
			case "/cmd/system/get-environment-data" : 
				content = ServerCommandSystem.cmd_get_environment_data( _css, get_data_map );
				break;

			case "/cmd/system/get-cube-data" : 
				content = ServerCommandSystem.cmd_get_cube_data( _css, get_data_map );
				break;
				
			case "/cmd/system/get-status" : 
				content = ServerCommandSystem.cmd_get_status( _css, get_data_map );
				break;

			case "/cmd/system/get-result" :				
				content = ServerCommandSystem.cmd_get_result( _css, get_data_map );
				break;
						
			//ServerCommandProject
			case "/cmd/project/get-projects" : 
				content = ServerCommandProject.cmd_get_projects( _css, get_data_map );
				break;
				
			case "/cmd/project/create-project" : 
				content = ServerCommandProject.cmd_create_project( _css, get_data_map );
				break;

			case "/cmd/project/open-project" :
				content = ServerCommandProject.cmd_open_project( _css, get_data_map );
				break;

			case "/cmd/project/get-current-project" :
				content = ServerCommandProject.cmd_get_current_project( _css, get_data_map );
				break;

			case "/cmd/project/open-projects-directory" :
				content = ServerCommandProject.cmd_open_projects_directory( _css, get_data_map );
				break;

			//ServerCommandRepository
			case "/cmd/repository/scan-repositories" :
				content = ServerCommandRepository.cmd_scan_repositories( _css, get_data_map );
				break;
				
			case "/cmd/repository/get-main-lists" :
				content = ServerCommandRepository.cmd_get_main_lists( _css, get_data_map );
				break;			

			case "/cmd/repository/get-sections" :
				content = ServerCommandRepository.cmd_get_sections( _css, get_data_map );
				break;

			case "/cmd/repository/get-section-packages" :
				content = ServerCommandRepository.cmd_get_section_packages( _css, get_data_map );
				break;

			case "/cmd/repository/get-available-packages" :
				content = ServerCommandRepository.cmd_get_available_packages( _css, get_data_map );
				break;

			case "/cmd/repository/get-installed-packages" :
				content = ServerCommandRepository.cmd_get_installed_packages( _css, get_data_map );
				break;

			case "/cmd/repository/get-upgradable-packages" :
				content = ServerCommandRepository.cmd_get_upgradable_packages( _css, get_data_map );
				break;

			case "/cmd/repository/get-downloaded-packages" :
				content = ServerCommandRepository.cmd_get_downloaded_packages( _css, get_data_map );
				break;			

			case "/cmd/repository/get-package-dependencies" :
				content = ServerCommandRepository.cmd_get_package_dependencies( _css, get_data_map );
				break;
			
			case "/cmd/repository/get-satisfied-downloaded-packages" :
				content = ServerCommandRepository.cmd_get_satisfied_downloaded_packages( _css, get_data_map );
				break;

			case "/cmd/repository/get-next" :
				content = ServerCommandRepository.cmd_get_next( _css, get_data_map );
				break;

			case "/cmd/repository/get-previous" :
				content = ServerCommandRepository.cmd_get_previous( _css, get_data_map );
				break;

			case "/cmd/repository/find-package" :
				content = ServerCommandRepository.cmd_find_package( _css, get_data_map );
				break;

			case "/cmd/repository/find-package-starts-with" :
				content = ServerCommandRepository.cmd_find_package_starts_with( _css, get_data_map );
				break;

			case "/cmd/repository/get-marked-packages" :
				content = ServerCommandRepository.cmd_get_marked_packages( _css, get_data_map );
				break;

			case "/cmd/repository/mark-package" :
				content = ServerCommandRepository.cmd_mark_package( _css, get_data_map );
				break;

			case "/cmd/repository/unmark-package" :
				content = ServerCommandRepository.cmd_unmark_package( _css, get_data_map );
				break;

			//ServerCommandDownload
			case "/cmd/download/download-package" :
				content = ServerCommandDownload.cmd_download_package( _css, get_data_map );
				break;

			case "/cmd/download/download-repository" :
				content = ServerCommandDownload.cmd_download_repository( _css, get_data_map );
				break;

			case "/cmd/download/stop-download" :
				content = ServerCommandDownload.cmd_download_stop( _css, get_data_map );
				break;

			case "/cmd/download/get-download-package-result" :
				content = ServerCommandDownload.cmd_get_download_package_result( _css, get_data_map );
				break;

			case "/cmd/download/get-download-repository-result" :
				content = ServerCommandDownload.cmd_get_download_repository_result( _css, get_data_map );
				break;

			case "/cmd/download/get-download-status" : 
				content = ServerCommandDownload.cmd_get_download_status( _css, get_data_map );
				break;

			//ServerCommandInstall
			case "/cmd/install/install-package" :
				content = ServerCommandInstall.cmd_install_package( _css, get_data_map );
				break;

			case "/cmd/install/get-install-package-result" :
				content = ServerCommandInstall.cmd_get_install_package_result( _css, get_data_map );
				break;

			//ServerCommandUpdate
			case "/cmd/update/update-project" :
				content = ServerCommandUpdate.cmd_update_project( _css, get_data_map );
				break;

			case "/cmd/update/get-update-project-result" :
				content = ServerCommandUpdate.cmd_get_update_project_result( _css, get_data_map );
				break;
				
			case "/cmd/update/update-system" :
				content = ServerCommandUpdate.cmd_update_system( _css, get_data_map );
				break;

			case "/cmd/update/get-update-system-result" :
				content = ServerCommandUpdate.cmd_get_update_system_result( _css, get_data_map );
				break;

			//ServerCommandSource
			case "/cmd/source/add-ppa-repository" :
				content = ServerCommandSource.add_ppa_repository( _css, get_data_map );
				break;

			case "/cmd/source/check-ppa-exists" :
				content = ServerCommandSource.check_ppa_exists( _css, get_data_map );
				break;

			//ServerCommandConfiguration
			case "/cmd/configuration/get-configuration-entries" :
				content = ServerCommandConfiguration.cmd_get_configuration_entries( _css, get_data_map );
				break;

			case "/cmd/configuration/save-configuration-entries" :
				content = ServerCommandConfiguration.cmd_save_configuration_entries( _css, get_data_map );
				break;
			
			case "/cmd/configuration/open-sources-list-file" :
				content = ServerCommandConfiguration.cmd_open_sources_list_file( _css, get_data_map );
				break;

			case "/cmd/bye" :
				stop_server_requested ();
				content = "Camicri Cube Server is now closed. Good Bye :)";
				break;
		
		}
		
		response_occurred ( content.data, content.length, "text/plain" );
	}

	public void route_assets ( string location, TreeMap<string,string> get_data_map )
	{
		string type = "";
		uint8[] content_data = null;
				
		if ( location.has_prefix("/assets/js") )
		{
			content_data = ServerAssets.assets_js ( _css, get_data_map ).data;
			type = "text/javascript";				
		}
		else if ( location.has_prefix("/assets/css") )
		{
			content_data = ServerAssets.assets_css ( _css, get_data_map ).data;
			type = "text/css";
		}
		else if ( location.has_prefix("/assets/images") )
		{
			content_data = ServerAssets.assets_images ( _css, get_data_map );
			type = "image";
		}
		else if ( location.has_prefix("/assets/html") )
		{
			content_data = ServerAssets.assets_html ( _css, get_data_map ).data;
			type = "text/html";
		}
		else if ( location.has_prefix("/assets/fonts") )
		{
			content_data = ServerAssets.assets_fonts ( _css, get_data_map );
			type = "application/octet-stream";
		}

		response_occurred ( content_data , content_data.length, type );
	}

}
