/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* cube-server
 *
 * Copyright (C) Jake R. Capangpangan 2015 <camicrisystems@gmail.com>
 *
cube-server is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cube-server is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using Gee;

public class CubeServer
{
	SocketService _service;
	SocketConnection _conn;
	OutputStream _output_stream;

	bool _stop_flag = false;

	public signal void started ();
	public signal void stopped ();
	public signal void error_occurred ( string message );
	public signal void request_occurred ( string location, TreeMap<string,string> get_data_map );	
	
	public CubeServer()
	{		
	}

	public bool use_port(uint16 port)
	{
		try 
		{
		    _service = new SocketService ();
		    _service.add_inet_port (port, null);
		}
		catch (Error e) 
		{
		    stdout.printf("Error : %s\n",e.message);
			error_occurred ( e.message );
			return false;
		}
		
		return true;
	}

	public bool start_server()
	{
		try
		{			
			_stop_flag = false;			
			_service.start ();
			started();
			
			while (true) 
			{				
		   		_conn = _service.accept (null);
		   		bool alive = process_request (_conn.input_stream, _conn.output_stream);
				if ( ! alive || _stop_flag )
					break;
			}			
		}
		catch (Error e) 
		{
		    stderr.printf ("%s\n", e.message);
			error_occurred ( e.message );
			return false;
		}	

		stopped();
		return true;
	}

	public void stop_server()
	{		
		_service.stop();
		_stop_flag = true;
	}
	
	public bool process_request(InputStream input, OutputStream output) throws Error 
	{
		var data_in = new DataInputStream (input);
		_output_stream = output;
		
		string line;
		TreeMap<string,string> data_map = new TreeMap<string,string>();
		
		while ((line = data_in.read_line (null)) != null) 
		{
		    if (line.strip () == "") break;

			if ( line.has_prefix ("GET") )
			{
				data_map.clear();
				data_map = get_data_map( line );				
			}
		}
		
		request_occurred ( data_map["location"], data_map );
		
		return true;
	}

	//TODO Enable POST data extraction too
	public TreeMap<string,string> get_data_map ( owned string request_data )
	{
		string url = request_data;		
		
		TreeMap<string,string> data_map = new TreeMap<string,string>( );
		if ( request_data.has_prefix ("GET") ) //GET request
		{
			url = request_data.split(" ",3)[1];
			if ( url.contains("?") )
			{
				string location = url.split("?",2)[0];
				url = url.split("?",2)[1];
				
				if ( url.contains("&") )
				{
					foreach ( string vars in url.split("&",-1) )
						data_map[vars.split("=",2)[0]] = Uri.unescape_string(vars.split("=",2)[1]);					
				}
				else
					data_map[url.split("=",2)[0]] = Uri.unescape_string(url.split("=",2)[1]);

				data_map["location"] = location;
			}
			else
				data_map["location"] = url.strip();
			
		}		

		return data_map;
	}

	/*
	 * Headers
	 * HTML - text/html
	 * Image - image/png
	 * Text - text/plain
	 * Javascript/JSON - text/javascript , application/javascript
	 * CSS - text/css 
	 * */	
	public bool write ( uint8[] content_data, int content_length, string type )
	{
		try
		{			
			var header = new StringBuilder ();
			header.append ("HTTP/1.0 200 OK\r\n");
			//This header fixes json and ajax problems! :)
			// Allow CORS
			header.append ("Access-Control-Allow-Origin: *\r\n");
			header.append ("Content-Type: "+type.strip()+"\r\n");			
			header.append_printf ("Content-Length: %lu\r\n\r\n", content_data.length);			

			_output_stream.write (header.str.data);
			_output_stream.write (content_data);
			_output_stream.flush();
			//This fixes response problems to firefox
			_output_stream.close();
				
		}
		catch ( Error e )
		{
			//stdout.printf("Error : %s\n",e.message);
			return false;
		}
		return true;
	}
}